import React, { useEffect, useState } from 'react'

import { Button } from '@/components/ui/button'
import { Input } from '@/components/ui/input'
import { Checkbox } from '@/components/ui/checkbox'
import { toast } from 'sonner'

import {
  CheckBeforeInit,
  InitSQLiteHelper,
  QueryMacAddressByPrefix,
  QueryVersion,
} from '../../wailsjs/go/service/SQLiteHelper'
import { GetLatestVersion } from '../../wailsjs/go/service/UpdateService'
import { ReloadDatabase } from '../../wailsjs/go/service/SQLiteHelper'
import {
  AlertDialog,
  AlertDialogAction,
  AlertDialogCancel,
  AlertDialogContent,
  AlertDialogDescription,
  AlertDialogFooter,
  AlertDialogHeader,
  AlertDialogTitle,
  AlertDialogTrigger,
} from '@/components/ui/alert-dialog'

import Download from './Download'

const Panel: React.FC = () => {
  const [macInputDisabled, setMacInputDisabled] = useState(false)

  const [macAddress, setMacAddress] = useState('')
  const [prefix, setPrefix] = useState('')
  const [vendorName, setVendorName] = useState('')
  const [isPrivate, setIsPrivate] = useState(false)
  const [blockType, setBlockType] = useState('')
  const [lastUpdate, setLastUpdate] = useState('')
  const [databaseDate, setDatabaseDate] = useState('')

  const [checkDisabled, setCheckDisabled] = useState(false)

  const [newVersion, setNewVersion] = useState('')
  const [isOpen, setIsOpen] = useState(false)
  const [isDownloading, setIsDownloading] = useState(false)

  useEffect(() => {
    CheckBeforeInit().then(res => {
      if (res) {
        InitSQLiteHelper().then(res => {
          if (res) {
            QueryVersion().then(res => {
              setDatabaseDate(res)
            })
          } else {
            toast('Init dababase failed.', {
              description: '⛓️‍💥 Please check your internet connection.',
              action: {
                label: 'Close',
                onClick: () => {},
              },
            })
          }
        })
      } else {
        setCheckDisabled(true)
        setMacInputDisabled(true)
        // download
      }
    })
  }, [])

  const handleTextChange = (e: React.ChangeEvent<HTMLInputElement>) => {
    setMacAddress(e.target.value)
    if (e.target.value === '') {
      setPrefix('')
      setVendorName('')
      setIsPrivate(false)
      setBlockType('')
      setLastUpdate('')
      return
    }
    let address = e.target.value
      .trim()
      .toUpperCase()
      .replace(/[:\-\.]/g, '')
    if (address.length < 6) {
      return
    }
    address =
      address.substring(0, 2) + ':' + address.substring(2, 4) + ':' + address.substring(4, 6)
    QueryMacAddressByPrefix(address).then(res => {
      if (res === null) {
        setPrefix('Not Found')
        setVendorName('')
        setIsPrivate(false)
        setBlockType('')
        setLastUpdate('')
        return
      }
      setPrefix(res.Prefix)
      setVendorName(res.VendorName)
      setIsPrivate(res.IsPrivate)
      setBlockType(res.BlockType)
      setLastUpdate(res.LastUpdate)
    })
  }

  const handleCheck = () => {
    setCheckDisabled(true)
    GetLatestVersion().then(res => {
      if (res.Version === null) {
        setCheckDisabled(false)
        toast('Check update failed.', {
          description: '⛓️‍💥 Please check your internet connection.',
          action: {
            label: 'Close',
            onClick: () => {},
          },
        })
      } else {
        setCheckDisabled(false)
        if (res === databaseDate) {
          toast('No new version available', {
            description: '🎉 You are using the latest version.',
            action: {
              label: 'Close',
              onClick: () => {},
            },
          })
        } else {
          // alert
          setNewVersion(res)
          setIsOpen(true)
        }
      }
    })
  }

  return (
    <div className="mx-auto p-6">
      <div className="mb-2 flex items-center">
        <Input
          className="p-1 border h-8"
          value={macAddress}
          onChange={handleTextChange}
          placeholder="Enter MAC Address"
          disabled={macInputDisabled}
        />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Prefix</label>
        <Input className="w-2/3 p-1 border h-8" value={prefix} readOnly />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Vendor</label>
        <Input className="w-2/3 p-1 border h-8" value={vendorName} readOnly />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Private</label>
        <Checkbox className="w-4 h-4" checked={isPrivate} disabled />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Block Type</label>
        <Input className="w-2/3 p-1 border h-8" value={blockType} readOnly />
      </div>
      <div className="mb-2 flex items-center">
        <label className="w-1/3 select-none">Last Update</label>
        <Input className="w-2/3 p-1 border h-8" value={lastUpdate} readOnly />
      </div>
      <div className="flex items-center">
        <label className="w-1/3 select-none">Mac Database</label>
        <Input className="w-5/12 p-1 border h-8" value={databaseDate} readOnly />
        <Button
          className="ml-2 p-1 h-8 w-1/4 select-none"
          onClick={handleCheck}
          disabled={checkDisabled}
        >
          Check
        </Button>
      </div>
      <AlertDialog open={isOpen}>
        <AlertDialogContent>
          <AlertDialogHeader>
            <AlertDialogTitle>Update Available</AlertDialogTitle>
            <AlertDialogDescription>
              Version {newVersion} is available. Do you want to download it now?
            </AlertDialogDescription>
          </AlertDialogHeader>
          <AlertDialogFooter>
            <AlertDialogCancel
              onClick={() => {
                setIsOpen(false)
              }}
            >
              No
            </AlertDialogCancel>
            <AlertDialogAction
              onClick={() => {
                setIsOpen(false)
                setIsDownloading(true)
              }}
            >
              Yes
            </AlertDialogAction>
          </AlertDialogFooter>
        </AlertDialogContent>
      </AlertDialog>
      {isDownloading && (
        <Download
          onFinish={() => {
            ReloadDatabase().then(res => {
              if (res) {
                QueryVersion().then(res => {
                  setDatabaseDate(res)
                })
              }
            })
          }}
        />
      )}
    </div>
  )
}

export default Panel
